package sun.util.locale;

import java.util.HashMap;
import java.util.Map;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Locale;
import java.util.List;

public final class LocaleMatcher
{
    public static List<Locale> filter(List<Locale.LanguageRange> list, Collection<Locale> collection, Locale.FilteringMode filteringMode) {
        if (list.isEmpty() || collection.isEmpty()) {
            return new ArrayList();
        }
        ArrayList<String> list2 = new ArrayList();
        Iterator<Object> iterator = collection.iterator();
        while (iterator.hasNext()) {
            list2.add(((Locale)iterator.next()).toLanguageTag());
        }
        List<String> filterTags = filterTags(list, list2, filteringMode);
        ArrayList o = new ArrayList(filterTags.size());
        Iterator<Object> iterator2 = filterTags.iterator();
        while (iterator2.hasNext()) {
            o.add(Locale.forLanguageTag((String)iterator2.next()));
        }
        return o;
    }
    
    public static List<String> filterTags(List<Locale.LanguageRange> list, Collection<String> collection, Locale.FilteringMode filteringMode) {
        if (list.isEmpty() || collection.isEmpty()) {
            return new ArrayList();
        }
        if (filteringMode == Locale.FilteringMode.EXTENDED_FILTERING) {
            return filterExtended(list, collection);
        }
        ArrayList<Locale.LanguageRange> list2 = new ArrayList();
        for (Locale.LanguageRange e : list) {
            String range = e.getRange();
            if (range.startsWith("*-") || range.indexOf("-*") != -1) {
                if (filteringMode == Locale.FilteringMode.AUTOSELECT_FILTERING) {
                    return filterExtended(list, collection);
                }
                if (filteringMode == Locale.FilteringMode.MAP_EXTENDED_RANGES) {
                    String replaceAll;
                    if (range.charAt(0) == '*') {
                        replaceAll = "*";
                    }
                    else {
                        replaceAll = range.replaceAll("-[*]", "");
                    }
                    list2.add(new Locale.LanguageRange(replaceAll, e.getWeight()));
                }
                else {
                    if (filteringMode == Locale.FilteringMode.REJECT_EXTENDED_RANGES) {
                        throw new IllegalArgumentException("An extended range \"" + range + "\" found in REJECT_EXTENDED_RANGES mode.");
                    }
                    continue;
                }
            }
            else {
                list2.add(e);
            }
        }
        return filterBasic(list2, collection);
    }
    
    private static List<String> filterBasic(List<Locale.LanguageRange> list, Collection<String> c) {
        ArrayList<String> list2 = new ArrayList();
        Iterator<Object> iterator = list.iterator();
        while (iterator.hasNext()) {
            String range = ((Locale.LanguageRange)iterator.next()).getRange();
            if (range.equals("*")) {
                return new ArrayList(c);
            }
            Iterator<Object> iterator2 = c.iterator();
            while (iterator2.hasNext()) {
                String lowerCase = ((String)iterator2.next()).toLowerCase();
                if (lowerCase.startsWith(range)) {
                    int length = range.length();
                    if ((lowerCase.length() == length || lowerCase.charAt(length) == '-') && !list2.contains(lowerCase)) {
                        list2.add(lowerCase);
                    }
                    else {
                        continue;
                    }
                }
                else {
                    continue;
                }
            }
        }
        return list2;
    }
    
    private static List<String> filterExtended(List<Locale.LanguageRange> list, Collection<String> c) {
        ArrayList<String> list2 = new ArrayList();
        Iterator<Object> iterator = list.iterator();
        while (iterator.hasNext()) {
            String range = ((Locale.LanguageRange)iterator.next()).getRange();
            if (range.equals("*")) {
                return new ArrayList(c);
            }
            String[] split = range.split("-");
            Iterator<Object> iterator2 = c.iterator();
            while (iterator2.hasNext()) {
                String lowerCase = ((String)iterator2.next()).toLowerCase();
                String[] split2 = lowerCase.split("-");
                if (!split[0].equals(split2[0]) && !split[0].equals("*")) {
                    continue;
                }
                int n = 1;
                int n2 = 1;
                while (n < split.length && n2 < split2.length) {
                    if (split[n].equals("*")) {
                        ++n;
                    }
                    else if (split[n].equals(split2[n2])) {
                        ++n;
                        ++n2;
                    }
                    else {
                        if (split2[n2].length() == 1 && !split2[n2].equals("*")) {
                            break;
                        }
                        ++n2;
                    }
                }
                if (split.length == n && !list2.contains(lowerCase)) {
                    list2.add(lowerCase);
                }
                else {
                    continue;
                }
            }
        }
        return list2;
    }
    
    public static Locale lookup(List<Locale.LanguageRange> list, Collection<Locale> collection) {
        if (list.isEmpty() || collection.isEmpty()) {
            return null;
        }
        ArrayList<String> list2 = new ArrayList();
        Iterator<Object> iterator = collection.iterator();
        while (iterator.hasNext()) {
            list2.add(((Locale)iterator.next()).toLanguageTag());
        }
        String lookupTag = lookupTag(list, list2);
        if (lookupTag == null) {
            return null;
        }
        return Locale.forLanguageTag(lookupTag);
    }
    
    public static String lookupTag(List<Locale.LanguageRange> list, Collection<String> collection) {
        if (list.isEmpty() || collection.isEmpty()) {
            return null;
        }
        Iterator<Object> iterator = list.iterator();
        while (iterator.hasNext()) {
            String range = ((Locale.LanguageRange)iterator.next()).getRange();
            if (range.equals("*")) {
                continue;
            }
            String regex = range.replaceAll("\\x2A", "\\\\p{Alnum}*");
            while (regex.length() > 0) {
                Iterator<Object> iterator2 = collection.iterator();
                while (iterator2.hasNext()) {
                    String lowerCase = ((String)iterator2.next()).toLowerCase();
                    if (lowerCase.matches(regex)) {
                        return lowerCase;
                    }
                    continue;
                }
                int lastIndex = regex.lastIndexOf(45);
                if (lastIndex >= 0) {
                    regex = regex.substring(0, lastIndex);
                    if (regex.lastIndexOf(45) == regex.length() - 2) {
                        regex = regex.substring(0, regex.length() - 2);
                    }
                    else {
                        continue;
                    }
                }
                else {
                    regex = "";
                }
            }
        }
        return null;
    }
    
    public static List<Locale.LanguageRange> parse(String s) {
        s = s.replaceAll(" ", "").toLowerCase();
        if (s.startsWith("accept-language:")) {
            s = s.substring(16);
        }
        String[] split = s.split(",");
        ArrayList o = new ArrayList(split.length);
        ArrayList o2 = new ArrayList();
        int n = 0;
        for (String substring : split) {
            int index;
            double double1;
            if ((index = substring.indexOf(";q=")) == -1) {
                double1 = 1.0;
            }
            else {
                substring = substring.substring(0, index);
                index += 3;
                try {
                    double1 = Double.parseDouble(substring.substring(index));
                }
                catch (Exception ex) {
                    throw new IllegalArgumentException("weight=\"" + substring.substring(index) + "\" for language range \"" + substring + "\"");
                }
                if (double1 < 0.0 || double1 > 1.0) {
                    throw new IllegalArgumentException("weight=" + double1 + " for language range \"" + substring + "\". It must be between " + 0.0 + " and " + 1.0 + ".");
                }
            }
            if (!o2.contains(substring)) {
                Locale.LanguageRange o3 = new Locale.LanguageRange(substring, double1);
                int j = 0;
                while (j < n) {
                    if (((Locale.LanguageRange)o.get(j)).getWeight() < double1) {
                        n = j;
                        break;
                    }
                    ++j;
                }
                o.add(n, o3);
                ++n;
                o2.add(substring);
                String equivalentForRegionAndVariant;
                if ((equivalentForRegionAndVariant = getEquivalentForRegionAndVariant(substring)) != null && !o2.contains(equivalentForRegionAndVariant)) {
                    o.add(n + 1, new Locale.LanguageRange(equivalentForRegionAndVariant, double1));
                    ++n;
                    o2.add(equivalentForRegionAndVariant);
                }
                String[] equivalentsForLanguage;
                if ((equivalentsForLanguage = getEquivalentsForLanguage(substring)) != null) {
                    for (String range : equivalentsForLanguage) {
                        if (!o2.contains(range)) {
                            o.add(n + 1, new Locale.LanguageRange(range, double1));
                            ++n;
                            o2.add(range);
                        }
                        String equivalentForRegionAndVariant2 = getEquivalentForRegionAndVariant(range);
                        if (equivalentForRegionAndVariant2 != null && !o2.contains(equivalentForRegionAndVariant2)) {
                            o.add(n + 1, new Locale.LanguageRange(equivalentForRegionAndVariant2, double1));
                            ++n;
                            o2.add(equivalentForRegionAndVariant2);
                        }
                    }
                }
            }
        }
        return o;
    }
    
    private static String[] getEquivalentsForLanguage(String substring) {
        while (substring.length() > 0) {
            if (LocaleEquivalentMaps.singleEquivMap.containsKey(substring)) {
                return new String[] { substring.replaceFirst(substring, (String)LocaleEquivalentMaps.singleEquivMap.get(substring)) };
            }
            if (LocaleEquivalentMaps.multiEquivsMap.containsKey(substring)) {
                String[] array = (String[])LocaleEquivalentMaps.multiEquivsMap.get(substring);
                for (int i = 0; i < array.length; ++i) {
                    array[i] = substring.replaceFirst(substring, array[i]);
                }
                return array;
            }
            int lastIndex = substring.lastIndexOf(45);
            if (lastIndex == -1) {
                break;
            }
            substring = substring.substring(0, lastIndex);
        }
        return null;
    }
    
    private static String getEquivalentForRegionAndVariant(String s) {
        int extentionKeyIndex = getExtentionKeyIndex(s);
        for (String s2 : LocaleEquivalentMaps.regionVariantEquivMap.keySet()) {
            int index;
            if ((index = s.indexOf(s2)) != -1) {
                if (extentionKeyIndex != Integer.MIN_VALUE && index > extentionKeyIndex) {
                    continue;
                }
                int index2 = index + s2.length();
                if (s.length() == index2 || s.charAt(index2) == '-') {
                    return s.replaceFirst(s2, (String)LocaleEquivalentMaps.regionVariantEquivMap.get(s2));
                }
                continue;
            }
            else {
                continue;
            }
        }
        return null;
    }
    
    private static int getExtentionKeyIndex(String s) {
        char[] charArray = s.toCharArray();
        int n = Integer.MIN_VALUE;
        for (int i = 1; i < charArray.length; ++i) {
            if (charArray[i] == '-') {
                if (i - n == 2) {
                    return n;
                }
                n = i;
            }
        }
        return Integer.MIN_VALUE;
    }
    
    public static List<Locale.LanguageRange> mapEquivalents(List<Locale.LanguageRange> c, Map<String, List<String>> map) {
        if (c.isEmpty()) {
            return new ArrayList();
        }
        if (map == null || map.isEmpty()) {
            return new ArrayList(c);
        }
        HashMap<String, String> hashMap = new HashMap();
        for (String s : map.keySet()) {
            hashMap.put(s.toLowerCase(), s);
        }
        ArrayList o = new ArrayList();
        for (Locale.LanguageRange languageRange : c) {
            String s2 = languageRange.getRange();
            boolean n = false;
            while (s2.length() > 0) {
                if (hashMap.containsKey(s2)) {
                    n = true;
                    List<Object> list = (List)map.get(hashMap.get(s2));
                    if (list != null) {
                        int length = s2.length();
                        Iterator<Object> iterator3 = list.iterator();
                        while (iterator3.hasNext()) {
                            o.add(new Locale.LanguageRange(((String)iterator3.next()).toLowerCase() + s2.substring(length), languageRange.getWeight()));
                        }
                        break;
                    }
                    break;
                }
                else {
                    int lastIndex = s2.lastIndexOf(45);
                    if (lastIndex == -1) {
                        break;
                    }
                    s2 = s2.substring(0, lastIndex);
                }
            }
            if (n == 0) {
                o.add(languageRange);
            }
            else {
                continue;
            }
        }
        return o;
    }
    
    private LocaleMatcher() {
    }
}
